function [xstar,k,rc,W,lambda]=qpin(Q,c,A,b,kmax,epz)
% minimize (1/2)x'Qx+c'x subject to Ax<=b

% initialize
  n=size(Q,1);                      % number of variables
  m=size(A,1);                      % number of inequalities
  tol=1e-6;                         % zero tolerance

% find a feasible starting point
  xzero=zeros(n,1);                 % use origin if unconstrained
  if(m > 0)                         % if there are constraints
     [xzero,rcf]=feas(A,b);         % get a feasible starting point
     if(rcf ~= 0)                   % success?
        rc=4;                       % no; signal failure
        return                      % and give up
     end                            % feasible point has been found
     OK=false;                      % active set has not been found
  end
  W=zeros(1,m); OK=true;            % working set starts empty
  Abar=zeros(0,n); mbar=0;          % active A starts empty
  lambda=zeros(m,1);                % multipliers start zero
  xk=xzero;                         % now have initial xk, W, Abar

  rc=1;                             % in case of nonconvergence
  for k=1:kmax
%     find reduced Newton direction
      if(mbar == n)                 % if active constraints square
         rc=0;                      % signal success
         break                      % and return unique solution
      elseif(mbar == 0)             % subproblem is unconstrained
         Z=eye(n);                  % Z=I makes Z'*Q*Z=Q
      else                          % 0 < mbar < n
         Z=null(Abar);              % get a basis for the nullspace
      end                           % now have Z
      [U,rch]=hfact(Z'*Q*Z,0.5);    % factor the reduced Hessian
      if(rch ~= 0)                  % success?
         rc=3;                      % report modification failed
         break                      % and give up
      end                           % now Z'QZ=U'U
      V=Z/U;                        % solve VU=Z for V
      Hinv=V*V';                    % find Hinv=Z*inv(Z'QZ)*Z'
      d=-Hinv*(Q*xk+c);             % full reduced Newton step

%     find step length
      alpha=1;                      % full step if no constraints
      for i=1:m                     % examine each constraint
          if(W(i) == 0)                       % assumed inactive?
             if(A(i,:)*d <= 0) continue; end  % increasing slack OK
             if(abs(A(i,:)*xk-b(i)) < tol)    % if already tight
                alpha=0;                        % cannot tighten
                break                           % no move possible
             else                             % move decreases slack
                r=(b(i)-A(i,:)*xk)/(A(i,:)*d);  % maximum step
                r=max(r,0);                     % can't be negative
                alpha=min(alpha,r);             % make alpha no more
             end                              % this constraint done
          end                       % ignored constraints checked
      end                           % now have alpha

%     take reduced Newton step
      xk=xk+alpha*d;                % take the step

      OK=true;                      % assume W will not change
      if(mbar > 0)                  % any constraints active?
%        find Lagrange multipliers
         g=Q*xk+c;                        % objective gradient
         [lambda,rcg]=getlgm(m,Abar,W,g); % use formula
         if(rcg ~= 0)                     % was there trouble?
            rc=2;                         % report Abar*Abar' not pd
            break                         % and give up
         end                        % now have multipliers

%        release sticking constraints
         for i=1:m                  % examine each constraint
             if(W(i) == 1)          % is it active?
                if(lambda(i)<-tol)  % yes; is it sticking?
                   W(i)=0;          % sticking; make it inactive
                   OK=false;        % not converged yet
                end
             end
         end                        % sticking constraints released
      end

%     find constraint values and activate blocking constraints
      Abar=zeros(0,n);              % active A empty
      mbar=0;                       % assume no active constraints
      for i=1:m                     % examine each constraint
          if(W(i) == 1)             % is it in the working set?
             Abar=[Abar;A(i,:)];    % already active; copy its A row
             mbar=mbar+1;           % and count it
          else                             % not in working set
             if(mbar == n) break; end      % no more than n active
             if(abs(A(i,:)*xk-b(i)) < tol) % if it is tight
                if(A(i,:)*d > 0)           % and move would violate
                   W(i)=1;             % it is blocking; activate it
                   OK=false;           % not converged yet
                   mbar=mbar+1;        % increase the count
                   Abar=[Abar;A(i,:)]; % add the row to Abar
                end
             end
          end
      end                           % blocking constraints active

%     test for convergence
      if(OK && norm(d)<epz)         % W unchanged and step small?
         rc=0;                      % signal success
         break                      % and return
      end                           % convergence tested
  end
  xstar=xk;                         % return the current point

end
