% naive Wolfe line search based on bisection
function [astar,rc,s]=wolfe(xk,dk,xl,xh,n,fcn,grd,mu,eta,tol,smax)

% initialize, and check for sensible inputs
  s=0;         % before searching we have done no iterations
  astar=0;     % and xk is the best point we know
  fk=fcn(xk);  % this is the function value there
  gk=grd(xk);  % this is the gradient vector there
  dfk=gk'*dk;  % this is the directional derivative in direction dk
  if(dfk == 0) % does the function descend in either direction?
     if(norm(gk) == 0)
        rc=0;  % no because xk is a stationary point
        return
     else
        rc=5;  % no because dk is orthogonal to gk
        return
     end
  end
  [amin,amax]=arange(xk,dk,xl,xh,n);
  if(amin > 0)
     rc=6;     % xk is not in [xl,xh] so no interval to search
     return
  end
  if(mu <= 0 || mu >= 1 || eta <= 0 || eta >= 1)
     rc=7;     % at least one Wolfe parameter has an illegal value
     return
  end
  a=0;         % the line search will start from xk
  xa=xk;       % that is the left end of the search interval
  fa=fk;       % this the function value there
  fr=fa;       % before searching it is the best value we know
  if(dfk < 0)  % which direction is downhill?
     c=amax;   % descend towards amax
  else
     c=amin;   % descend towards amin
  end
  rc=4;        % prepare to report failure

% search the interval [a,c] for the minimizing point
  for s=1:smax
      b=(a+c)/2;             % find the midpoint value of alpha
      xb=xk+b*dk;            % find the midpoint value of x
      gb=grd(xb);            % find the gradient there
      dfb=gb'*dk;            % find the directional derivative there
      if(dfb == 0)           % is this point exactly stationary?
        rc=0;                % yes; inform the caller
        astar=b;             % and return it
        return
      end

%     check the Wolfe conditions
      fb=fcn(xb);            % find the function value at midpoint
      if(fb <= fk+mu*dfk*b)  % check sufficient decrease condition
         if(abs(dfb) <= eta*abs(dfk))  % check curvature condition
            rc=2;            % this is a Wolfe point
            if(fb < fr)      % is this the best point found so far?
               fr=fb;        % yes; update the record value
               astar=b;      % save the record point for return
               if(abs(dfb) < tol)      % is it stationary enough?
                  rc=1;      % the Wolfe point also satisfies tol
                  return     % return it
               end
            end
         end
      else
%        the function did not decrease enough; halve the step
         c=b;
         fc=fb;
         continue
      end

%     decide which half to keep and bisect the interval
      if(dfb < 0)
         a=b;                % the minimum is between b and c
         fa=fb;
      else
         c=b;                % the minimum is between a and b
         fc=fb;
      end
      if(a == c) break; end
  end

% out of iterations or search interval has shrunk to zero
  if(rc == 2)
%    astar is the best Wolfe point but |f'(astar)| >= tol
     return
  else
     astar=b;                % return the final non-Wolfe point
     if(abs(dfb) < tol)      % is it at least stationary enough?
        rc=3;                % yes; report that
        return
     else
        return               % no; return with rc=4 set above
     end
  end
end
